#---------------------------------------ASSIGNMENT III--------------------------
# Authors: TsungWei CHENG     - 478595
#          Eren CAVUSOGLU     - 472075
#          Marlon THEIS       - 463921
#          Mustafa ATICI      - 480325
#          Mounami BEERGONDAR - 472086
rm(list = ls())

library(timelineS)
library(lubridate)
library(ggplot2)
library(dplyr)
library(reshape2)
library(rPref)
library(mco)
library(MASS)

plot.timeline <- function(lifetime, events.name, start.Data, plot.Name){
  dataP <- data.frame(Events = events.name, Event_Datas = ymd(start.Data) + 
                        years(lifetime))
  timelineS(dataP, main = plot.Name, 
            labels = events.name, label.direction = "up", label.position = 3)
}

dist.Events <- function(lifetime, events, start.Date, option.Name){
  #sort the events
  events <- sort(events, decreasing = T)
  #create the distribution of the events
  distribution.events <- sapply(events, seq, from = 0, to = lifetime)
  #all events unlisted
  all.events <- melt(distribution.events)
  colnames(all.events) <- c("frequency", "event")
  #sort the events ascending
  all.events <- all.events[order(all.events$frequency),]
  #get the unique sequence of events
  unique.events <- all.events[!duplicated(all.events$frequency),]
  #Initial construction of classrooms(CR)
  unique.events$event[which(unique.events$frequency == 0)] <- "CR"  
  #plot the timeline
  #plot.timeline(unique.events$frequency, unique.events$event, start.Date, option.Name)
  return(unique.events)
}

# We have three products, facades, a corridor, and ceiling and slab 
# that needs to be integrated into six classrooms. 
# ============================================================
# Product.1 specifies the facades. 
# two design options are of facades.
#
# CBMF 180 mm thickness is of cement, cast in place concrete,
#                    and concrete block, steel reinforcement, 
#
# PRC 240 mm thickness is of cement, precast concrete, and
#                                    steel reinforcement.
# the cement inventory can be found in TsungWei's and Mounami's
# papers
#
# the cast in place concrete inventory can be found in TsungWei's
# and Marlon's papers which depends on how details we want to do
#
# steel reinforcement inventory can be found in TsungWei's, Marlon's,
# and Mounami's papers
# 
# concrete block inventory is provided in a paper uploaded in drive, 
# additional paper for inventory, around page 50. The title is 
# "Life-Cycle-Inventory-of-Concrete.pdf"
# 
# precast concrete inventory is shown in TsungWei's paper. 
#
# =============================================================
# Product.2 is the corridors
# Chosen is Cast in place pavement, and information concerns this 
# has provided in Mounami's paper
#
# ==============================================================
# Product.3 concerns the ceiling and slab
# Two options is selected WCC and CLT, which the information is 
# shown in Marlon's paper
#
#

combine.lifeTimelines <- function(product.1, product.2, product.3, p1.dur, p2.dur, p3.dur)
{
  #function body
  base <- list(Names = c(), frequency = c(), duration = c())
  base$frequency <- sort(unique(c(product.1$frequency, 
                                  product.2$frequency,
                                  product.3$frequency)),
                         decreasing = F)
  base$Names[1] <- "CR"
  base$duration[1] <- 0
  base$Names[length(base$frequency)] <- "END"
  base$duration[length(base$frequency)] <- 0
  
  for(index in 2:(length(base$frequency) - 1)){
    phase.1 <- product.1$event[which(product.1$frequency == 
                                       base$frequency[index])]
    phase.2 <- product.2$event[which(product.2$frequency ==
                                       base$frequency[index])]
    phase.3 <- product.3$event[which(product.3$frequency ==
                                      base$frequency[index])]
    if(length(phase.1)!= 0 && length(phase.2)!= 0 && length(phase.3) != 0){
      base$Names[index] <- paste(phase.1, phase.2, phase.3)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)],
                                  p3.dur[which(names(p3.dur) == phase.3)])
    }else if(length(phase.1)!= 0 && length(phase.2)!= 0 && length(phase.3) == 0){
      base$Names[index] <- paste(phase.1,phase.2)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p2.dur[which(names(p2.dur) == phase.2)])
    }
    else if(length(phase.1)!= 0 && length(phase.2) == 0 && length(phase.3) != 0){
      base$Names[index] <- paste(phase.1, phase.3)
      base$duration[index] <- max(p1.dur[which(names(p1.dur) == phase.1)],
                                  p3.dur[which(names(p3.dur) == phase.3)])
    }else if(length(phase.1)== 0 && length(phase.2)!= 0 && length(phase.3) != 0){
      base$Names[index] <- paste(phase.2, phase.3)
      base$duration[index] <- max(p2.dur[which(names(p2.dur) == phase.2)],
                                  p3.dur[which(names(p3.dur) == phase.3)])
    }else if(length(phase.1) != 0 && length(phase.2) == 0 && length(phase.3) == 0){
      base$Names[index] <- phase.1
      base$duration[index] <- p1.dur[which(names(p1.dur) == phase.1)]
    }else if(length(phase.1) == 0 && length(phase.2) != 0 && length(phase.3) == 0){
      base$Names[index] <- phase.2
      base$duration[index] <- p2.dur[which(names(p2.dur) == phase.2)]
    }else if(length(phase.1) == 0 && length(phase.2) == 0 && length(phase.3) != 0){
      base$Names[index] <- phase.3
      base$duration[index] <- p3.dur[which(names(p3.dur) == phase.3)]
    }else{
      base$duration[index] <- 0
    }
  }
  return(base)
}

par(mfrow = c(2,1))
start.Date = "2023-01-01"
lifetime <- 100
#
# facade
# M.C is concrete structure preventive maintenance every 22 years, and duration is 17 days
# R.C is concrete structure replacement every 80 years, and duration is 30 days
# M.PC is maintenance prefab concrete panels every 20 years, and duration is 10 days
# R.PC is prefab concrete replacement every 50 years, and duration is 22 days
#
# ==================================================================
#
# corridor
# RH.PV is rehabilitation for pavement every 12 years, and duration is 20 days
# M.PV is preventive maintenance every 10 years, and duration is 7 days
# R.PV is pavement replacement every 40 years, and duration is 40 days
#
# ==================================================================
#
# ceiling and slab
# M.WCC is preventive maintenance for WCC every 12 years, and duration is 10 days
# R.WCC is rehabilitation for WCC every 50 years, and duration is 24 days
# M.CLT is preventive maintenance for CLT every 10 years, and duration is 7 days
# R.CLT is rehabilitation for CLT every 50 years, and duration is 22 days
#
#        facade         corridor         ceiling and slab
# Op1    CBMF           pavement         WCC
# Op2    CBMF           pavement         CLT
# Op3    PRC            pavement         WCC
# Op4    PRC            pavement         CLT
#

events.OpCBMF.facade <- c(M.C = 22, R.C = 80, END = lifetime)
duration.evCBMF.facade <- c(M.C = 17, R.C = 30)
events.OpPRC.facade <- c(M.PC = 20, R.PC = 50, END = lifetime)
duration.evPRC.facade <- c(M.PC = 10, R.PC = 22)
events.Op.corridor <- c(RH.PV = 12, M.PV = 10, R.PV = 40, END = lifetime)
duration.ev.Corridor <- c(RH.PV = 20, M.PV = 7, R.PV = 40)
events.OpWCC.CeilingAndSlab <- c(M.WCC = 12, R.WCC = 50, END = lifetime)
duration.evWCC.CeilingAndSlab <- c(M.WCC = 10, R.WCC = 24)
events.OpCLT.CeilingAndSlab <- c(M.CLT = 10, R.CLT = 50, END = lifetime)
duration.evCLT.CeilingAndSlab <- c(M.CLT = 7, R.CLT = 22)

design.OptionsCBMF.facade <- list(design.OpCBMF.facade <- "CBMF facade")
design.OptionsPRC.facade <- list(design.OpPRC.facade <- "PRC facade")
design.Options.corridor <- list(design.Op.corridor <- "Pavement")
design.OptionsWCC.CeilingAndSlab <- list(design.OpWCC.CeilingAndSlab <- "WCC Ceiling And Slab")
design.optionsCLT.CeilingAndSlab <- list(design.OpCLT.CeilingAndSlab <- "CLT Ceiling And Slab")

maintenance.OpCBMF.facade <- dist.Events(
  lifetime, events.OpCBMF.facade, start.Date, design.OptionsCBMF.facade)
maintenance.OpPRC.facade <- dist.Events(
  lifetime, events.OpPRC.facade, start.Date, design.OptionsPRC.facade)
maintenance.Op.corridor <- dist.Events(
  lifetime, events.Op.corridor, start.Date, design.Options.corridor)
maintenance.OpWCC.CeilingAndSlab <- dist.Events(
  lifetime, events.OpWCC.CeilingAndSlab, start.Date, design.OptionsWCC.CeilingAndSlab)
maintenance.OpCLT.CeilingAndSlab <- dist.Events(
  lifetime, events.OpCLT.CeilingAndSlab, start.Date, design.optionsCLT.CeilingAndSlab)

"
test
maintenance.Op.corridor$frequency
maintenance.OpCBMF.facade$frequency
maintenance.OpWCC.CeilingAndSlab$frequency
duration.ev.Corridor
duration.evCBMF.facade
duration.evWCC.CeilingAndSlab

base <- list(Names = c(), frequency = c(), duration = c())
base$frequency <- sort(unique(c(maintenance.OpCBMF.facade$frequency, 
                                maintenance.Op.corridor$frequency,
                                maintenance.OpWCC.CeilingAndSlab$frequency)),
                       decreasing = F)

phase.2 <- maintenance.Op.corridor$event[which(maintenance.Op.corridor$frequency ==
                                   base$frequency[9])]

phase.2
maintenance.Op.corridor$event
"


integrated.interv.Op1 <- combine.lifeTimelines(maintenance.OpCBMF.facade, 
                                           maintenance.Op.corridor,
                                           maintenance.OpWCC.CeilingAndSlab,
                                           duration.evCBMF.facade,
                                           duration.ev.Corridor,
                                           duration.evWCC.CeilingAndSlab)

integrated.interv.Op2 <- combine.lifeTimelines(maintenance.OpCBMF.facade,
                                               maintenance.Op.corridor,
                                               maintenance.OpCLT.CeilingAndSlab,
                                               duration.evCBMF.facade,
                                               duration.ev.Corridor,
                                               duration.evCLT.CeilingAndSlab)

integrated.interv.Op3 <- combine.lifeTimelines(maintenance.OpPRC.facade,
                                               maintenance.Op.corridor,
                                               maintenance.OpWCC.CeilingAndSlab,
                                               duration.evPRC.facade,
                                               duration.ev.Corridor,
                                               duration.evWCC.CeilingAndSlab)

integrated.interv.Op4 <- combine.lifeTimelines(maintenance.OpPRC.facade,
                                               maintenance.Op.corridor,
                                               maintenance.OpCLT.CeilingAndSlab,
                                               duration.evPRC.facade,
                                               duration.ev.Corridor,
                                               duration.evCLT.CeilingAndSlab)

###
integrated.interv.Op1
integrated.interv.Op2
integrated.interv.Op3
integrated.interv.Op4
###

sum(integrated.interv.Op1$duration)    
sum(integrated.interv.Op2$duration)
sum(integrated.interv.Op3$duration)
sum(integrated.interv.Op4$duration)

# design Option 4 cost the fewest days for duration, 
# the following data will use the option4 as the design option


design.Optimal.classroom <- list(
  design.Optimal.classroom <- "PRC facades, pavement, and CLT slab and ceiling classroom")

maintenance.Optimal.classroom <- plot.timeline(
  integrated.interv.Op4$frequency, integrated.interv.Op4$Names, start.Date, design.Optimal.classroom)

design.explore <- function(event1, event2, event3)
{
  #close plot.timeline
  results <- c()
  for(i in 1:dim(event1)[1]){
    ev1 <- unlist(event1[i, ])
    dist.1 <- dist.Events(lifetime, ev1, start.Date, "facade")
    dur.ev1 <- ev1 / 2
    for(j in 1:dim(event1)[1]){
      ev2 <- unlist(event2[j, ])
      dist.2 <- dist.Events(lifetime, ev2, start.Date, "Corridor")
      dur.ev2 <- ev2 / 2
      for (k in 1:dim(event1)[1]){
        ev3 <- unlist(event3[k, ])
        dist.3 <- dist.Events(lifetime, ev3, start.Date, "CeilingAndSlab")
        dur.ev3 <- ev3 / 2
        combined.lifetime <- combine.lifeTimelines(dist.1, dist.2, dist.3,
                                                   dur.ev1, dur.ev2, dur.ev3)
        min.dist.int <- min(abs(combined.lifetime$frequency[1 : (length(combined.lifetime$frequency) - 1)] - 
                                  combined.lifetime$frequency[2 : (length(combined.lifetime$frequency))]))
        results <- rbind(results, c(ev1, ev2, ev3, dur = sum(combined.lifetime$duration),
                                    dist.inter = min.dist.int))
      }
    }
  }
  return(as.data.frame(results))
}
#--------------------------Durations-------------------------------------------------
#
# facade
# M.PC is maintenance prefab concrete panels every 20 years, and duration is 10 days
# R.PC is prefab concrete replacement every 50 years, and duration is 22 days
# range for interventions
# 17 <= M.PC <= 23
# 45 <= R.PC <= 55
#
# corridor
# RH.PV is rehabilitation for pavement every 12 years, and duration is 20 days
# M.PV is preventive maintenance every 10 years, and duration is 7 days
# R.PV is pavement replacement every 40 years, and duration is 40 days
# range for interventions
# 7 < RH.PV < 15
# 8 < M.PV < 12
# 37 < R.PV < 43
#
# Ceiling and slab
# M.CLT is preventive maintenance for CLT every 10 years, and duration is 7 days
# R.CLT is rehabilitation for CLT every 50 years, and duration is 22 days
# range for interventions
# 7 < M.CLT < 12
# 47 < R.CLT < 52
#

n.grid <- 2

events.grid.facade <- expand.grid(M.PC = sample(seq(17, 23), n.grid),
                                  R.PC = sample(seq(45, 55), n.grid))

events.grid.corridor <- expand.grid(RH.PV = sample(seq(7, 15), n.grid),
                                    M.PV = sample(seq(8, 12), n.grid),
                                    R.PV = sample(seq(37, 43), n.grid))

events.grid.CeilingAndSlab <- expand.grid(M.CLT = sample(seq(7,12), n.grid),
                                          R.CLT = sample(seq(47, 52), n.grid))

response.space <- design.explore(events.grid.facade, events.grid.corridor, events.grid.CeilingAndSlab)
response.space

p <- low(dur) * high(dist.inter)
p
####
try.low <- low(dur)
try.high <- high(dist.inter)
####
sky <- psel(response.space, p)
sky

pareto2 <- psel(response.space, p, top = nrow(response.space))

ggplot(response.space, aes(x = dur, y = dist.inter)) +
  geom_point(shape = 21) +
  geom_point(data = pareto2, size = 3, aes(color = factor(pareto2$.level)))

show_front <- function(pref) {
  plot(response.space$dur, response.space$dist.inter)
  sky <- psel(response.space, pref)
  plot_front(response.space, pref, col = rgb(0, 0, 1))
  points(sky$dur, sky$dist.inter, lwd = 3)
}
show_front(p)

p <- high(dur) * low(dist.inter)
show_front(p)
#----------------------------------UPLOAD CSV FILE/change the path--------------
LCI.materials <- read.csv("Material_integrated.csv")

LCA.classroom <- function(Length, Width, Quantity, height, Components.Options, 
                          materials, dist.event)
{
  materials.split <- split(materials, materials$scope)
  #lines for intervention
  s <- summary(as.factor(dist.event$event))
  s2 <- as.data.frame(rbind(Freq = s), stringsAsFactors = F, row.names = 1:length(s))
  print(s2)
  Wall.thickness <- 0.24
  #facade options
  if(Components.Options == "PRC")
    {
    #intervention.walls <- s2$
    # there are two sides walls, and four short walls minus to both sides of wall.thickness
    # x -> 2 * (Length + 2 * Width)
    # After that wall height times wall.thickness equal to section, where x times sections
    # is the volume for single classroom row.
    # Therefore, two row mean quantity equal to two
    walls.volume <- 2 * (Length + 2 * (Width - 2 * Wall.thickness)) * Wall.thickness * height * Quantity
    intervention.wall <- s2$CR + 0.5 * s2$R.PC + 0.35 * (if(!is.null(s2$M.PC)){s2$M.PC} 
                                                   else{0})
    walls <- mutate(materials.split[[Components.Options]], Classroom.Q = walls.volume, 
                    interventions = intervention.wall)
    LCA.matrix <- walls
  }
  else if (Components.Options == "CBMF"){
    walls.volume <- 2 * (Length + 2 * (Width - 2 * Wall.thickness)) * Wall.thickness * height * Quantity
    intervention.wall <- s2$CR + 0.45 * s2$R.C + 0.5 * (if(!is.null(s2$M.C)){s2$M.C}
                                                        else{0})
    walls <- mutate(materials.split[[Components.Options]], Classroom.Q = walls.volume, 
                    interventions = intervention.wall)
    LCA.matrix <- walls
  }
  #corridor options
  if(Components.Options == "Pavement")
  {
    subbase.volume <- 0.34 * Length * Width * height
    surface.volume <- 0.18 * Length * Width * height
    asphalt.volume <- Length * Width * 0.18
    intervention.Cor <- s2$CR + 0.35 * s2$R.PV + 
      0.5 * (if(!is.null(s2$RH.PV)){s2$RH.PV}else{0}) + 0.25 * (if(!is.null(s2$M.PV)){s2$M.PV}else{0})
    intervention.asphalt <- length(dist.event$event)-1
    Components.Options <- "Binder Surface"
    subbase <- mutate(materials.split[[Components.Options]], Classroom.Q = subbase.volume,
                      interventions = intervention.Cor)
    Components.Options <- "Base Sub"
    surface <- mutate(materials.split[[Components.Options]], Classroom.Q = subbase.volume,
                      interventions = intervention.Cor)
    asphalt <- mutate(materials.split$asphalt, Classroom.Q = asphalt.volume, 
                      interventions = intervention.asphalt)
    LCA.matrix <- rbind(subbase, surface, asphalt)
  }
  #Ceiling And Slab
  if(Components.Options == "CLT"){
    Panel.volume <- Length * Width * height * 2 * Quantity
    intervention.Panel <- s2$CR + 0.25 * s2$R.CLT + 0.35 * (if(!is.null(s2$PV.CLT)){s2$PV.CLT}
                                                            else{0})
    panel <- mutate(materials.split[[Components.Options]], Classroom.Q = Panel.volume,
                    interventions = intervention.Panel)
    LCA.matrix <- panel
  }else if(Components.Options == "WCC"){
    Panel.volume <- Length * Width * height * 2 * Quantity
    intervention.Panel <- s2$CR + 0.24 * s2$R.WCC + 0.35 * (if(!is.null(s2$PV.WCC)){s2$PV.WCC}
                                                            else{0})
    panel <- mutate(materials.split[[Components.Options]], Classroom.Q = Panel.volume,
                    interventions = intervention.Panel)
    LCA.matrix <- panel
  }
  
  LCA.matrix <- mutate(LCA.matrix, TotalMaterials.Q = quantities * Classroom.Q / 1000,
                       materials.LC = TotalMaterials.Q * interventions,
                       Energy.LC = materials.LC * energy,
                       CO2.LC = materials.LC * CO2 * 1000,
                       NOx.LC = materials.LC * NOx * 1000,
                       SO2.LC = materials.LC * SO2 * 1000)

  LCA.results <- list(Energy = sum(LCA.matrix$Energy.LC),
                      CO2 = sum(LCA.matrix$CO2.LC),
                      NOx = sum(LCA.matrix$NOx.LC),
                      SO2 = sum(LCA.matrix$SO2.LC))
  return(LCA.results)
}

facade.Options <- c("PRC", "CBMF")
corridor.Options <- c("Pavement")
CeilingAndSlab.Options <- c("CLT", "WCC")

#unit:m
CR.length <- 24.50 
CR.width <- 6.30 
CR.quantity <- 2
CR.height <- 4.30

Cor.width <- 6.00
Cor.thickness <- 0.4
Cor.quantity <- 1

PL.thickness <- 15

Options.Facades.PRC <- LCA.classroom(CR.length, CR.width, CR.quantity, CR.height, 
                                 facade.Options[1], LCI.materials, maintenance.OpPRC.facade)
Options.Facades.CBMF <- LCA.classroom(CR.length, CR.width, CR.quantity, CR.height, 
                                      facade.Options[2], LCI.materials, maintenance.OpCBMF.facade)
Options.Corridor <- LCA.classroom(CR.length, Cor.width, Cor.quantity, Cor.thickness,
                                  corridor.Options[1], LCI.materials, maintenance.Op.corridor)
Options.CeilingAndSlab.CLT <- LCA.classroom(CR.length, CR.width, CR.quantity, PL.thickness, 
                                            CeilingAndSlab.Options[1], LCI.materials, maintenance.OpCLT.CeilingAndSlab)
Options.CeilingAndSlab.WCC <- LCA.classroom(CR.length, CR.width, CR.quantity, PL.thickness, 
                                            CeilingAndSlab.Options[2], LCI.materials, maintenance.OpWCC.CeilingAndSlab)

#        facade         corridor         ceiling and slab
# Op1    CBMF           pavement         WCC
# Op2    CBMF           pavement         CLT
# Op3    PRC            pavement         WCC
# Op4    PRC            pavement         CLT
#

integrated.design.Op1 <- as.data.frame(list(Energy = Options.Facades.CBMF$Energy + Options.Corridor$Energy + Options.CeilingAndSlab.WCC$Energy,
                                            CO2 = Options.Facades.CBMF$CO2 + Options.Corridor$CO2 + Options.CeilingAndSlab.WCC$CO2,
                                            NOx = Options.Facades.CBMF$NOx + Options.Corridor$NOx + Options.CeilingAndSlab.WCC$NOx,
                                            SO2 = Options.Facades.CBMF$SO2 + Options.Corridor$SO2 + Options.CeilingAndSlab.WCC$SO2))

integrated.design.Op2 <- as.data.frame(list(Energy = Options.Facades.CBMF$Energy + Options.Corridor$Energy + Options.CeilingAndSlab.CLT$Energy,
                                            CO2 = Options.Facades.CBMF$CO2 + Options.Corridor$CO2 + Options.CeilingAndSlab.CLT$CO2,
                                            NOx = Options.Facades.CBMF$NOx + Options.Corridor$NOx + Options.CeilingAndSlab.CLT$NOx,
                                            SO2 = Options.Facades.CBMF$SO2 + Options.Corridor$SO2 + Options.CeilingAndSlab.CLT$SO2))

integrated.design.Op3 <- as.data.frame(list(Energy = Options.Facades.PRC$Energy + Options.Corridor$Energy + Options.CeilingAndSlab.WCC$Energy,
                                            CO2 = Options.Facades.PRC$CO2 + Options.Corridor$CO2 + Options.CeilingAndSlab.WCC$CO2,
                                            NOx = Options.Facades.PRC$NOx + Options.Corridor$NOx + Options.CeilingAndSlab.WCC$NOx,
                                            SO2 = Options.Facades.PRC$SO2 + Options.Corridor$SO2 + Options.CeilingAndSlab.WCC$SO2))

integrated.design.Op4 <- as.data.frame(list(Energy = Options.Facades.PRC$Energy + Options.Corridor$Energy + Options.CeilingAndSlab.CLT$Energy,
                                            CO2 = Options.Facades.PRC$CO2 + Options.Corridor$CO2 + Options.CeilingAndSlab.CLT$CO2,
                                            NOx = Options.Facades.PRC$NOx + Options.Corridor$NOx + Options.CeilingAndSlab.CLT$NOx,
                                            SO2 = Options.Facades.PRC$SO2 + Options.Corridor$SO2 + Options.CeilingAndSlab.CLT$SO2))
Energy.cost <- 0.128
CO2.unitCost <- 26
NOx.unitCost <- 42
SO2.unitCost <- 85

integrated.design.Op1 <- mutate(integrated.design.Op1, 
                                Costs = (Energy * Energy.cost +
                                           CO2 * CO2.unitCost +
                                           NOx * NOx.unitCost +
                                           SO2 * SO2.unitCost) / 10 ^ 9)

integrated.design.Op2 <- mutate(integrated.design.Op2, 
                                Costs = (Energy * Energy.cost +
                                           CO2 * CO2.unitCost +
                                           NOx * NOx.unitCost +
                                           SO2 * SO2.unitCost) / 10 ^ 9)

integrated.design.Op3 <- mutate(integrated.design.Op3, 
                                Costs = (Energy * Energy.cost +
                                           CO2 * CO2.unitCost +
                                           NOx * NOx.unitCost +
                                           SO2 * SO2.unitCost) / 10 ^ 9)

integrated.design.Op4 <- mutate(integrated.design.Op4, 
                                Costs = (Energy * Energy.cost +
                                           CO2 * CO2.unitCost +
                                           NOx * NOx.unitCost +
                                           SO2 * SO2.unitCost) / 10 ^ 9)

#
integrated.design.Op1
integrated.design.Op2
integrated.design.Op3
integrated.design.Op4
#

#--------------------------Durations_fitness-------------------------------------------------
#Fourth Design Option
# facade
# M.PC is maintenance prefab concrete panels every 20 years, and duration is 10 days
# R.PC is prefab concrete replacement every 50 years, and duration is 22 days
# range for interventions
# 17 <= M.PC <= 23
# 45 <= R.PC <= 55
#
# corridor
# RH.PV is rehabilitation for pavement every 12 years, and duration is 20 days
# M.PV is preventive maintenance every 10 years, and duration is 7 days
# R.PV is pavement replacement every 40 years, and duration is 40 days
# range for interventions
# 7 < RH.PV < 15
# 8 < M.PV < 12
# 37 < R.PV < 43
#
# Ceiling and slab
# M.CLT is preventive maintenance for CLT every 10 years, and duration is 7 days
# R.CLT is rehabilitation for CLT every 50 years, and duration is 22 days
# range for interventions
# 7 < M.CLT < 12
# 47 < R.CLT < 52
#

fitness <- function(x) { 
  #define the output dimension 
  z <- numeric(7) 
  
  x <- round(x, 0) 
  y <- expand.grid(M.PC = x[1], R.PC = x[2], RH.PV = x[3], M.PV = x[4], R.PV = x[5], M.CLT = x[6], R.CLT = x[7]) 
  
  dur.ev1 <- unlist(y[1:2]  / 2) 
  dur.ev2 <- unlist(y[3:5]  / 2) 
  dur.ev3 <- unlist(y[6:7]  / 2)
  dist.1 <- apply(y[1:2], 1,  FUN = dist.Events, lifetime = lifetime, start.Date = start.Date) 
  dist.2 <- apply(y[3:5], 1,  FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  dist.3 <- apply(y[6:7], 1,  FUN = dist.Events, lifetime = lifetime, start.Date = start.Date)
  results <- combine.lifeTimelines(dist.1[[1]], dist.2[[1]], dist.3[[1]], dur.ev1, dur.ev2, dur.ev3)
  z[1] <- sum(results[["duration"]]) 
  z[2] <- -min(abs(results$frequency[1:(length(results$frequency) - 1)] - results$frequency[2:length(results$frequency)]))
  
  CR.width <- 6.30
  CR.length <- 24.50 
  CR.quantity <- 2
  CR.height <- 4.30
  Cor.width <- 6.00
  Cor.thickness <- 0.4
  Cor.quantity <- 1
  facade.Options <- c("PRC", "CBMF")
  corridor.Options <- c("Pavement")
  CeilingAndSlab.Options <- c("CLT", "WCC")
  Energy.cost <- 0.128
  CO2.unitCost <- 26
  NOx.unitCost <- 42
  SO2.unitCost <- 85
  #Fourth Integrated Design.Op4:   CBMF           pavement         WCC
  product.1.facade <- LCA.classroom(CR.length, CR.width, CR.quantity, CR.height, 
                                    facade.Options[1], LCI.materials, dist.1[[1]]) 
  product.2.corridor <- LCA.classroom(CR.length, Cor.width, Cor.quantity, Cor.thickness,
                                     corridor.Options[1], LCI.materials, dist.2[[1]])
  product.3.CeilingAndSlabs <- LCA.classroom(CR.length, CR.width, CR.quantity, PL.thickness, 
                                             CeilingAndSlab.Options[1], LCI.materials, dist.3[[1]])
  
  integrated.system <- as.data.frame(list(Energy = product.1.facade$Energy + product.2.corridor$Energy + product.3.CeilingAndSlabs$Energy,
                                          CO2 = product.1.facade$CO2 + product.2.corridor$CO2 + product.3.CeilingAndSlabs$CO2,
                                          NOx = product.1.facade$NOx + product.2.corridor$NOx + product.3.CeilingAndSlabs$NOx,
                                          SO2 = product.1.facade$SO2 + product.2.corridor$SO2 + product.3.CeilingAndSlabs$SO2))
  integrated.system <- mutate(integrated.system, 
                              Costs = (Energy * Energy.cost +
                                         CO2 * CO2.unitCost +
                                         NOx * NOx.unitCost +
                                         SO2 * SO2.unitCost) / 10 ^ 9)
  
  z[3] <- integrated.system$Energy  
  z[4] <- integrated.system$CO2  
  z[5] <- integrated.system$NOx  
  z[6] <- integrated.system$SO2  
  z[7] <- integrated.system$Costs
  return(z) 
}

r2 <- nsga2(fitness, idim = 7, odim = 7,            
            generations=5, popsize=4,            
            lower.bounds= c(17, 45, 7, 8, 37, 7, 47),            
            upper.bounds= c(23, 55, 15, 12, 43, 12, 52))

r2 <- nsga2(fitness, idim = 7, odim = 7,            
            generations=10, popsize=100,            
            lower.bounds= c(17, 45, 7, 8, 37, 7, 47),            
            upper.bounds= c(23, 55, 15, 12, 43, 12, 52))


r2Results <- as.data.frame(r2$value) 
outNames <- c("duration", "interv.dist", "energy", "co2", "nox", "so2", "cost" ) 
colnames(r2Results) <- outNames

pareto3 <- as.data.frame(paretoFront(r2)) 
colnames(pareto3) <- outNames

ggplot(r2Results, aes(x = duration, y = cost)) + geom_point(shape = 21) + 
  geom_point(data = pareto3, size = 3, color="red") + 
  geom_line(data = pareto3, color="blue")

input.params <- round(r2$par, 0) 
all.results <- cbind(input.params, r2Results, r2$pareto.optimal) 
colnames(all.results) <- c("M.PC", "R.PC", "RH.PV", 
                           "M.PV", "R.PV", "M.CLT", "R.CLT",
                           "duration", "interv.dist", 
                           "energy", "co2", "nox", "so2", "cost", "pareto")

par(mfrow = c(1,1)) 
parcoord(all.results[, 1:14], var.label = T, 
         col = ifelse(all.results$pareto == TRUE, "indianred", "skyblue2"), 
         lty = ifelse(all.results$pareto == TRUE, 1, 3),  
         lwd = ifelse(all.results$pareto == TRUE, 3, 1))



















